@@ -32,9 +32,23 @@ class ProcessHelper:
     def remove_protocol_if_existent(page: str) -> str:
         return page.replace("https://", "").replace("http://", "")
 
+    @staticmethod
+    def check_for_unfinished_work(rest_client: RestClient):
+        unfinished_work = rest_client.get_unfinished_work()
+        if len(unfinished_work) > 0:
+            logger.info("Looks like unfinished work was left after last shutdown. Will deregister with blocking for"
+                        " 24h")
+            for w in unfinished_work:
+                logger.info("Deregister currently in work page " + str(w))
+                rest_client.unregister_page_in_work_and_block_for_time(
+                    w['fkAnalysisRun']['index'], w['trancoPageEntry'],
+                    "Client crashed last time analysing this page")
+
     @staticmethod
     def prepare_and_run_analysis(backend_info: BackendInformation, analysis_id: int, virtual_display=None,
-                                 threading_stop_event: threading.Event = None, no_error_check: threading.Event = None):
+                                 threading_stop_event: threading.Event = None,
+                                 webdriver_initialization_error_check: threading.Event = None,
+                                 invalid_config_error_check=None):
         config_directory = None
         try:
             rc = RestClient(backend_info.host, backend_info.port, backend_info.token)
@@ -65,19 +79,19 @@ class ProcessHelper:
                 analysis_process = SSODetectionProcess(backend_info, analysis_id, analysis_run_type)
             elif analysis_run_type == ProcessType.PRIVACY_DETECTION:
                 from processes.privacydetection.privacy_detection_process import PrivacyDetectionProcess
-                from model.privacydetection.privacy_detection_type import PrivacyDetectionType
-                privacy_detection_type = PrivacyDetectionType[rc.get_privacy_detection_mode_for_run(analysis_id)]
                 analysis_process = PrivacyDetectionProcess(backend_info, analysis_id, analysis_run_type,
-                                                           privacy_detection_type,
                                                            config_directory)
+            elif analysis_run_type == ProcessType.SSO_SECURITY_ANALYSIS:
+                from processes.ssosecurityanalysis.sso_security_analysis_process import SSOSecurityProcess
+                analysis_process = SSOSecurityProcess(backend_info, analysis_id, analysis_run_type, config_directory)
             else:
                 raise TypeError("Unsupported analysis type: " + analysis_run_type.__str__())
             try:
-                analysis_process.start_process(running_check=threading_stop_event, no_error_check=no_error_check)
-                analysis_process.finish()
+                analysis_process.start_process(running_check=threading_stop_event,
+                                               webdriver_initialisation_error_check=webdriver_initialization_error_check,
+                                               invalid_config_error_check=invalid_config_error_check)
             except KeyboardInterrupt:
                 logger.info("\n\nQuitting... Closing services connection and driver")
-                analysis_process.finish()
         finally:
             if virtual_display is not None:
                 logger.info("Stopping virtual display")
@@ -158,24 +172,34 @@ class ProcessHelper:
         for request in chromedriver.requests:
             if request.url == site or request.url == site + "/":
                 site_found = True
-            if site_found and (300 <= request.response.status_code < 400 and follow_redirects):
+            if site_found and hasattr(request.response, "status_code") and (
+                    300 <= request.response.status_code < 400 and follow_redirects):
                 continue
+            elif site_found and not hasattr(request.response, "status_code"):
+                logger.warning("Looks like the request is running in endless mode or has canceled abnormally. "
+                               "No status_code for response was found.")
+                return -1
             elif site_found:
                 return request.response.status_code
-        logger.warning("WARNING! Could not find request for site " +
-                       site + ". Maybe http request was send but HSTS Header was set previously")
+        logger.warning("Could not find request for site " + site + (
+            ". Maybe http request was send but HSTS Header was set previously" if site.startswith(
+                "http://") else ". Maybe connection was canceled"))
         return -1
 
     @staticmethod
     def quit_chromedriver_correctly(chromedriver: webdriver):
         import urllib3
+        if hasattr(chromedriver, "killed"):
+            logger.info("Quitting chromedriver not needed as it was already quit")
+            return
         try:
             logger.info("Quitting chromedriver with " + str(len(chromedriver.window_handles)) + " windows")
             for handle in chromedriver.window_handles:
                 chromedriver.switch_to.window(handle)
                 chromedriver.close()
             chromedriver.quit()
-            del chromedriver.requests
+            logger.info("Done")
+            chromedriver.killed = True
         except urllib3.exceptions.MaxRetryError:
             logger.info("Looks like chromedriver was already closed (e.g. by interrupt signal)")
         except Exception as err:
