@@ -48,13 +48,19 @@ class DriverManager:
                                                   throw_exception_if_element_not_found)
 
     @staticmethod
-    def generate_driver(config_directory=None):
+    def generate_driver(config_directory=None, allow_running_insecure_content=False, remove_response_csp_headers=False,
+                        wait_for_page_to_load=True):
         try:
             options = webdriver.ChromeOptions()
             options.set_capability("unexpectedAlertBehaviour", "accept")
             options.set_capability("unhandledPromptBehavior", "accept")
+            if not wait_for_page_to_load:
+                options.set_capability("webdriver.load.strategy", "none")
+                options.set_capability("pageLoadStrategy", "none")
             if config_directory is not None:
                 options.add_argument("--user-data-dir=" + config_directory.name + "/chrome_profile")
+                if allow_running_insecure_content:
+                    options.add_argument("--allow-running-insecure-content")
                 extensions_file = open(config_directory.name + "/extensions.json")
                 extensions_meta_info = json.load(extensions_file)
                 for extension in extensions_meta_info['extensions']['packed']:
@@ -66,7 +72,6 @@ class DriverManager:
 
             options.add_experimental_option("excludeSwitches", ["enable-automation"])
             options.add_experimental_option('useAutomationExtension', False)
-
             DriverManager.download_extensions()
 
             for filename in os.listdir(extensions_folder):
@@ -80,18 +85,27 @@ class DriverManager:
             logger.info("Starting driver")
             driver = webdriver.Chrome(options=options, executable_path=ChromeDriverManager(log_level=0).install(),
                                       seleniumwire_options=selenium_wire_options)
+            if remove_response_csp_headers:
+                def interceptor(request, response):  # A response interceptor takes two args
+                    for header, val in response.headers.items():
+                        if header.lower() == "content-security-policy":
+                            del response.headers[header]
+
+                driver.response_interceptor = interceptor
+
             driver.implicitly_wait(1)
             driver.set_page_load_timeout(180)
 
             logger.info("Activate stealth for driver")
-            stealth(driver, languages=["en-US", "en"], vendor="Google Inc.", platform="Win32", webgl_vendor="Intel Inc.",
+            stealth(driver, languages=["en-US", "en"], vendor="Google Inc.", platform="Win32",
+                    webgl_vendor="Intel Inc.",
                     renderer="Intel Iris OpenGL Engine", fix_hairline=True)
             driver.set_window_position(0, 0)
             driver.set_window_size(1920, 1080)
-
+            logger.info("Driver initialisation finished")
             return driver
         except Exception as e:
-            logger.critical("An exception () occurred while initialising the driver!")
+            logger.critical("An exception (" + str(e.__class__.__name__) + ") occurred while initialising the driver!")
             raise exceptions.WebDriverInitialisationException(e)
 
     @staticmethod
