@@ -1,5 +1,6 @@
 import base64
 import json
+from time import sleep
 
 import requests
 
@@ -38,18 +39,25 @@ class RestClient:
     path_get_configuration = "/rest/analysisrun/configuration"
     # endregion
     # region SSO Detection Paths
-    path_get_ssodet_next_page_to_analyze = "/rest/ssodetection/nextPageToAnalyse"
+    path_get_next_page_to_detect_sso = "/rest/ssodetection/nextPageToAnalyse"
     path_save_sso_analysis = "/rest/ssodetection/new"
     # endregion
     # region Currently in Work Paths
+    path_unfinished_work = "/rest/currentlyinwork/unfinishedWork"
     path_unregister_currently_in_work_site_and_block = "/rest/currentlyinwork/unregisterCurrentlyInWorkAndBlockForTime"
     path_unregister_currently_in_work_site = "/rest/currentlyinwork/unregisterCurrentlyInWork"
     path_update_currently_in_work_site_progress = "/rest/currentlyinwork/reportProgress"
+    path_update_sso_security_analysis_progress = "/rest/currentlyinwork/reportSSOSecurityAnalysisProgress"
+    path_unregister_currently_in_work_sso_det = "/rest/currentlyinwork/unregisterCurrentlyInWorkSSODetection"
+    path_unregister_currently_in_work_sso_det_and_block = "/rest/currentlyinwork/unregisterCurrentlyInWorkSSODetectionAndBlockForTime"
     # endregion
     # region Privacy Detection
-    path_get_next_privacy_page_to_analyze = "/rest/privacy/nextPageToAnalyse"
+    path_get_next_privacy_page_to_analyze = "/rest/privacy/nextDetectionToAnalyse"
     path_save_privacy_detection = "/rest/privacy/new"
 
+    # region SSO Security Analysis
+    path_get_next_sso_detection_for_security_analysis = "/rest/ssosecurityanalysis/nextDetectionToAnalyse"
+    path_save_sso_security_analysis = "/rest/ssosecurityanalysis/new"
     # endregion
     # region Daemon
     path_daemon_start = "/rest/daemon/wakeup"
@@ -60,6 +68,9 @@ class RestClient:
 
     # endregion
     # endregion
+
+    retry_max_tries = 3
+
     def __init__(self, backend_url, backend_port, token=None):
         self.backend_url = backend_url
         self.port = backend_port
@@ -212,9 +223,15 @@ class RestClient:
         for id in ids:
             tid = [id[0]]
             if len(id) > 1:
-                tid.append(id[1]['x'])
-                tid.append(id[1]['y'])
-                tid.append(id[2])
+                if id[1] is not None and id[2] is not None:
+                    tid.append(id[1]['x'])
+                    tid.append(id[1]['y'])
+                    tid.append(id[2]['width'])
+                    tid.append(id[2]['height'])
+                else:
+                    tid.extend([None, None, None, None])
+                tid.append(id[3])
+                tid.append(id[4].name)
             transformed_ids.append(tid)
         post_data = {'analysisrunid': analysis_session, 'trancosite': trancosite,
                      'supportedproviderids': transformed_ids, "loginpages": []}
@@ -239,7 +256,7 @@ class RestClient:
             return False
 
     def get_next_ssodetection_page_to_analyse_for_run(self, analysis_run_id):
-        return self.get_next_page_to_analyse_for_run(self.path_get_ssodet_next_page_to_analyze, analysis_run_id)
+        return self.get_next_page_to_analyse_for_run(self.path_get_next_page_to_detect_sso, analysis_run_id)
 
     def get_search_engine_mode_for_run(self, run_id):
         return self.get_additional_analysis_run_information(run_id, "SEARCH_ENGINE")
@@ -247,8 +264,8 @@ class RestClient:
     # endregion
 
     # region Privacy Detection
-    def save_privacy_detection(self, trancosite, analysis_session, screenshot, har_content):
-        post_data = {'analysisrunid': analysis_session, 'trancosite': trancosite}
+    def save_privacy_detection(self, sso_det_index, analysis_session, screenshot, har_content):
+        post_data = {'analysisrunid': analysis_session, 'ssodetectionid': sso_det_index}
         files = {}
         if har_content is not None:
             files['har'] = har_content
@@ -264,10 +281,50 @@ class RestClient:
             return False
 
     def get_next_privacy_detection_page_to_analyse_for_run(self, analysis_run_id):
-        return self.get_next_page_to_analyse_for_run(self.path_get_next_privacy_page_to_analyze, analysis_run_id)
+        response = self.send_get_request(self.path_get_next_privacy_page_to_analyze, {'analysisrunid': analysis_run_id})
+        if response.status_code == 200:
+            if response.text is not None and len(response.text) != 0:
+                return response.json()
+            else:
+                return None  # No more sites left!
+        if response.status_code == 409:
+            logger.info("Server requested a retry")
+            raise RetryException()
+        logger.error("Something went wrong (code " + str(response.status_code) + ")")
 
-    def get_privacy_detection_mode_for_run(self, run_id):
-        return self.get_additional_analysis_run_information(run_id, "PRIVACY_DETECTION_TYPE")
+    # endregion
+
+    # region SSO Security Analysis
+    def get_next_sso_detection_for_security_analysis_for_run(self, analysis_run_id):
+        response = self.send_get_request(self.path_get_next_sso_detection_for_security_analysis,
+                                         {'analysisrunid': analysis_run_id})
+        if response.status_code == 200:
+            if response.text is not None and len(response.text) != 0:
+                return response.json()
+            else:
+                return None  # No more sites left!
+        if response.status_code == 409:
+            logger.info("Server requested a retry")
+            raise RetryException()
+        logger.error("Something went wrong (code " + str(response.status_code) + ")")
+
+    def save_sso_security_analysis(self, analysis_run_id, sso_detection_index: int, result: tuple, screen: bytes, har,
+                                   success=True, error=None):
+        post_data = {'analysisrunid': analysis_run_id, 'ssodetectionid': sso_detection_index, 'authreq': result[0],
+                     'authres': result[1], 'success': success, 'error': error}
+        files = {}
+        if har is not None:
+            files['har'] = har
+        if screen is not None:
+            files['screenshot'] = screen
+        response = self.send_post_request_with_files(self.path_save_sso_security_analysis, post_data,
+                                                     files)
+        if response.status_code == 200 and response.text == "success":
+            logger.info("Saving SSO Security analysis with HAR and Screenshot uploaded was successful!")
+            return True
+        else:
+            logger.error("Something went wrong (code " + str(response.status_code) + ")")
+            return False
 
     # endregion
 
@@ -288,6 +345,22 @@ class RestClient:
         logger.error("Something went wrong (" + str(response.status_code) + ")!")
         return False
 
+    def unregister_currently_in_work_sso_det(self, analysisrunid, detid):
+        response = self.send_post_request(self.path_unregister_currently_in_work_sso_det,
+                                          {'analysisrunid': analysisrunid, 'ssodetectionid': detid})
+        if response.status_code == 200:
+            return True
+        logger.error("Something went wrong (" + str(response.status_code) + ")")
+        return False
+
+    def unregister_currently_in_work_sso_det_and_block(self, analysisrunid, detid, cause="Unknown"):
+        response = self.send_post_request(self.path_unregister_currently_in_work_sso_det_and_block,
+                                          {'analysisrunid': analysisrunid, 'ssodetectionid': detid, 'cause': cause})
+        if response.status_code == 200:
+            return True
+        logger.error("Something went wrong (" + str(response.status_code) + ")")
+        return False
+
     def update_progress_of_analysis(self, analysisrunid: int, basepage: str, percent: int, status: str):
         response = self.send_post_request(self.path_update_currently_in_work_site_progress,
                                           {'analysisrunid': analysisrunid, 'basepage': basepage,
@@ -297,6 +370,22 @@ class RestClient:
         logger.error("Something went wrong updating progress status (" + str(response.status_code) + ")!")
         return False
 
+    def update_progress_of_analysis_using_sso_detection(self, analysis_run_id, sso_detection_id, percent: int, status: str):
+        response = self.send_post_request(self.path_update_sso_security_analysis_progress,
+                                          {'analysisrunid': analysis_run_id, 'ssodetectionid': sso_detection_id,
+                                           'percent': int(percent), 'status': status})
+        if response.status_code == 200:
+            return True
+        logger.error("Something went wrong updating progress status (" + str(response.status_code) + ")!")
+        return False
+
+    def get_unfinished_work(self):
+        response = self.send_get_request(self.path_unfinished_work)
+        if response.status_code == 200:
+            return response.json()
+        logger.error("Something went wrong ")
+        return False
+
     # endregion
 
     # region Analysis Runs
@@ -376,20 +465,59 @@ class RestClient:
 
     # region REQUEST METHODS
     def send_get_request(self, path, parameters=None):
-        if parameters is None:
-            return requests.get(self.url + path, headers=self.auth_bearer_header, allow_redirects=False)
-        else:
-            return requests.get(self.url + path, params=parameters, headers=self.auth_bearer_header,
-                                allow_redirects=False)
+        attempts_counter = 0
+        while attempts_counter < RestClient.retry_max_tries:
+            attempts_counter += 1
+            try:
+                if parameters is None:
+                    return requests.get(self.url + path, headers=self.auth_bearer_header, allow_redirects=False)
+                else:
+                    return requests.get(self.url + path, params=parameters, headers=self.auth_bearer_header,
+                                        allow_redirects=False)
+            except Exception as e:
+                self.handle_send_error(e, attempts_counter)
 
     def send_post_request(self, path, postdata):
-        return requests.post(self.url + path, data=postdata, headers=self.auth_bearer_header, allow_redirects=False)
+        attempts_counter = 0
+        while attempts_counter < RestClient.retry_max_tries:
+            attempts_counter += 1
+            try:
+                return requests.post(self.url + path, data=postdata, headers=self.auth_bearer_header,
+                                     allow_redirects=False)
+            except Exception as e:
+                self.handle_send_error(e, attempts_counter)
 
     def send_post_file_request(self, path, files):
-        logger.info("Size of fileupload: " + str(len(str(files)) * 0.000001) + " MB")
-        return requests.post(self.url + path, files=files, headers=self.auth_bearer_header, allow_redirects=False)
+        attempts_counter = 0
+        while attempts_counter < RestClient.retry_max_tries:
+            attempts_counter += 1
+            try:
+                logger.info("Size of fileupload: " + str(len(str(files)) * 0.000001) + " MB")
+                return requests.post(self.url + path, files=files, headers=self.auth_bearer_header,
+                                     allow_redirects=False)
+            except Exception as e:
+                self.handle_send_error(e, attempts_counter)
 
     def send_post_request_with_files(self, path, postdata, files):
-        return requests.post(self.url + path, data=postdata, files=files, headers=self.auth_bearer_header,
-                             allow_redirects=False)
+        attempts_counter = 0
+        while attempts_counter < RestClient.retry_max_tries:
+            attempts_counter += 1
+            try:
+                return requests.post(self.url + path, data=postdata, files=files, headers=self.auth_bearer_header,
+                                     allow_redirects=False)
+            except Exception as e:
+                self.handle_send_error(e, attempts_counter)
+
+    def handle_send_error(self, error, attempt):
+        if attempt < RestClient.retry_max_tries:
+            time_to_wait = 1 if attempt < (RestClient.retry_max_tries - 1) else 5
+            logger.error("Got an exception (" + str(
+                error.__class__.__name__) + ") when trying to send request. Will wait " + str(
+                time_to_wait) + " min and retry it")
+            sleep(time_to_wait * 60)
+        else:
+            logger.error("Could not send request after " + str(
+                RestClient.retry_max_tries) + " attempts. Will raise original error.")
+            raise error
+
     # endregion
